<?php

namespace QUI\Requirements\Tests\Webserver;

use QUI\Requirements\TestResult;
use QUI\Requirements\Tests\Test;

use function parse_url;

/**
 * Class Rewrite
 *
 * @package QUI\Requirements\Tests\Webserver
 */
class Rewrite extends Test
{
    /**
     * @var string
     */
    protected string $identifier = "webserver.rewrite";

    /**
     * @return TestResult
     */
    protected function run(): TestResult
    {
        if (php_sapi_name() == "cli") {
            return new TestResult(TestResult::STATUS_UNKNOWN);
        }

        if ($this->testRedirectByRequest()) {
            return new TestResult(TestResult::STATUS_OK);
        } else {
            return new TestResult(TestResult::STATUS_FAILED);
        }
    }

    /**
     * Executes a curl request against the server and checks if the request gets redirected properly.
     *
     * @return bool
     */
    protected function testRedirectByRequest()
    {
        if (php_sapi_name() == "cli") {
            return false;
        }

        $baseDir      = dirname(dirname(dirname(dirname(dirname(dirname(dirname(dirname(dirname(__FILE__)))))))));
        $htAccessPath = $baseDir . "/.htaccess";
        $serverUrl    = $_SERVER['HTTP_HOST'];

        if (defined('CMS_DIR')) {
            $baseDir = CMS_DIR;
        } elseif (isset($_SERVER['HTTP_REFERER'])) {
            $urlParts  = parse_url($_SERVER['HTTP_REFERER']);
            $serverUrl = $serverUrl . $urlParts['path'];
        }

        $serverUrl = $this->currentDir('http://' . $serverUrl);
        $serverUrl = str_replace('http://', '', $serverUrl);

        if (file_exists($htAccessPath)) {
            rename($htAccessPath, $htAccessPath . ".bak");
        }

        $checkValue = time();

        $htAccessContent = "RewriteEngine on" . PHP_EOL;
        $htAccessContent .= "RewriteRule ^rewritetest$ rewritetest.html [NC]" . PHP_EOL;

        file_put_contents($htAccessPath, $htAccessContent);
        file_put_contents($baseDir . "/rewritetest.html", $checkValue);

        // HTTP TEST
        $ch = curl_init('http://' . $serverUrl . "/rewritetest");
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

        $http_result = curl_exec($ch);
        curl_close($ch);

        // HTTPS TEST
        $ch = curl_init('https://' . $serverUrl . "/rewritetest");
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

        $https_result = curl_exec($ch);
        curl_close($ch);


        unlink($htAccessPath);
        unlink($baseDir . "/rewritetest.html");

        if (file_exists($htAccessPath . ".bak")) {
            rename($htAccessPath . ".bak", $htAccessPath);
        }

        if ($http_result == $checkValue || $https_result == $checkValue) {
            return true;
        }

        return false;
    }

    /**
     * @param $url
     * @return string
     *
     * found at https://stackoverflow.com/a/29147968
     * thx!
     */
    protected function currentDir($url): string
    {
        // note: anything without a scheme ("example.com", "example.com:80/", etc.) is a folder
        // remove query (protection against "?url=http://example.com/")
        if ($first_query = strpos($url, '?')) {
            $url = substr($url, 0, $first_query);
        }
        // remove fragment (protection against "#http://example.com/")
        if ($first_fragment = strpos($url, '#')) {
            $url = substr($url, 0, $first_fragment);
        }
        // folder only
        $last_slash = strrpos($url, '/');

        if (!$last_slash) {
            return '/';
        }
        // add ending slash to "http://example.com"
        if (($first_colon = strpos($url, '://')) !== false && $first_colon + 2 == $last_slash) {
            return $url . '/';
        }

        return substr($url, 0, $last_slash + 1);
    }
}
